#/bin/bash
zip -l ~kermit/archives/appleii.zip -n ps pdf * || exit 1
zip ~kermit/archives/appleii.zip *.ps *.pdf || exit 1
cd ~kermit/archives
ls -ldga appleii.zip || exit 1
unzip -t appleii.zip
exit 0
